"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const gpu = __importStar(require("gpu-compute"));
const limiter_1 = require("./limiter");
const uniforms_1 = require("./uniforms");
function bitonicSort(array, kind) {
    const bytes = new Uint8Array(array.buffer);
    const target = getRenderTarget(bytes);
    target.pushTextureData(bytes);
    const emptyTexelCount = target.width * target.width - bytes.length / 4;
    const generator = new uniforms_1.BitonicUniformGenerator(target, kind, emptyTexelCount);
    for (let x of generator.generate())
        target.compute(x.shader, x.uniforms);
    pullPixels(target, emptyTexelCount, bytes);
    target.delete();
    return array;
}
exports.bitonicSort = bitonicSort;
function bitonicSortAsync(array, kind) {
    return new Promise((resolve, reject) => {
        const bytes = new Uint8Array(array.buffer);
        const target = getRenderTarget(bytes);
        target
            .pushTextureDataAsync(bytes)
            .then(() => {
            const limiter = new limiter_1.Limiter(target.width);
            const emptyTexelCount = target.width * target.width - bytes.length / 4;
            const generator = new uniforms_1.BitonicUniformGenerator(target, kind, emptyTexelCount);
            for (let x of generator.generate())
                limiter.addWork(() => target.compute(x.shader, x.uniforms));
            limiter.onceFinished(() => {
                pullPixelsAsync(target, emptyTexelCount, bytes)
                    .then(() => resolve())
                    .catch(err => reject(err))
                    .finally(() => (target.delete(), limiter.stop()));
            });
        })
            .catch(err => (reject(err), target.delete()));
    });
}
exports.bitonicSortAsync = bitonicSortAsync;
function getRenderTarget(bytes) {
    const gl = gpu.getWebGLContext();
    const framebufferLimit = gl.getParameter(gl.MAX_RENDERBUFFER_SIZE);
    for (let width = 1; width <= framebufferLimit; width *= 2) {
        if (width * width * 4 >= bytes.length) {
            return new gpu.RenderTarget(width);
        }
    }
    throw new Error(`data overflows ${framebufferLimit}x${framebufferLimit} framebuffer`);
}
exports.getRenderTarget = getRenderTarget;
function pullPixels(target, e, bytes) {
    const w = target.width;
    const h = Math.floor(e / w);
    if (e % w > 0) {
        target.readSomePixels(e % w, h, w - (e % w), 1, bytes.subarray(0, 4 * (w - (e % w))));
        target.readSomePixels(0, h + 1, w, w - h - 1, bytes.subarray(4 * (w - (e % w))));
    }
    else {
        target.readSomePixels(0, h, w, w - h, bytes);
    }
}
exports.pullPixels = pullPixels;
function pullPixelsAsync(target, e, bytes) {
    return __awaiter(this, void 0, void 0, function* () {
        const w = target.width;
        const h = Math.floor(e / w);
        if (e % w > 0) {
            yield target.readSomePixelsAsync(e % w, h, w - (e % w), 1, bytes.subarray(0, 4 * (w - (e % w))));
            return target.readSomePixelsAsync(0, h + 1, w, w - h - 1, bytes.subarray(4 * (w - (e % w))));
        }
        else {
            return target.readSomePixelsAsync(0, h, w, w - h, bytes);
        }
    });
}
exports.pullPixelsAsync = pullPixelsAsync;
//# sourceMappingURL=bitonic.js.map