"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const gpu_compute_1 = require("gpu-compute");
class Limiter {
    constructor(width) {
        this.stopped = false;
        this.tally = 0;
        this.work = [];
        if (width >= 4096) {
            this.rate = 1;
        }
        else if (width === 2048) {
            this.rate = 4;
        }
        else if (width === 1024) {
            this.rate = 16;
        }
        else {
            this.rate = 128;
        }
        this.executeWork();
    }
    stop() {
        this.stopped = true;
    }
    executeWork() {
        this.tally += this.rate;
        if (this.work.length === 0) {
            this.tally = Math.max(this.tally, 1);
            if (!this.stopped)
                requestAnimationFrame(() => this.executeWork());
            return;
        }
        if (this.tally >= 1) {
            const limit = Math.floor(Math.min(this.work.length, Math.max(this.rate, 1)));
            for (let i = 0; i < limit; i++)
                this.work[i]();
            this.work.splice(0, limit);
            this.tally -= limit;
        }
        gpu_compute_1.getWebGLContext().flush();
        requestAnimationFrame(() => this.executeWork());
    }
    addWork(work) {
        if (this.stopped)
            throw new Error("limiter has been stopped");
        this.work.push(work);
    }
    onceFinished(callback) {
        const check = () => {
            if (!this.work.length) {
                callback();
            }
            else {
                requestAnimationFrame(() => check());
            }
        };
        check();
    }
}
exports.Limiter = Limiter;
//# sourceMappingURL=limiter.js.map