"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const init = __importStar(require("./initialize"));
var TRANSFORM_MODE;
(function (TRANSFORM_MODE) {
    TRANSFORM_MODE[TRANSFORM_MODE["PASSTHROUGH"] = 0] = "PASSTHROUGH";
    TRANSFORM_MODE[TRANSFORM_MODE["INTEGER"] = 1] = "INTEGER";
    TRANSFORM_MODE[TRANSFORM_MODE["FLOAT"] = 2] = "FLOAT";
})(TRANSFORM_MODE = exports.TRANSFORM_MODE || (exports.TRANSFORM_MODE = {}));
var CONSTRUCTOR_MODE;
(function (CONSTRUCTOR_MODE) {
    CONSTRUCTOR_MODE[CONSTRUCTOR_MODE["Int32Array"] = 0] = "Int32Array";
    CONSTRUCTOR_MODE[CONSTRUCTOR_MODE["Uint32Array"] = 1] = "Uint32Array";
    CONSTRUCTOR_MODE[CONSTRUCTOR_MODE["Float32Array"] = 2] = "Float32Array";
    CONSTRUCTOR_MODE[CONSTRUCTOR_MODE["Float64Array"] = 3] = "Float64Array";
    CONSTRUCTOR_MODE[CONSTRUCTOR_MODE["BigInt64Array"] = 4] = "BigInt64Array";
    CONSTRUCTOR_MODE[CONSTRUCTOR_MODE["BigUint64Array"] = 5] = "BigUint64Array";
})(CONSTRUCTOR_MODE = exports.CONSTRUCTOR_MODE || (exports.CONSTRUCTOR_MODE = {}));
function getBitonicParameters(constructorName) {
    switch (constructorName) {
        case "Int32Array":
            return {
                mode: TRANSFORM_MODE.INTEGER,
                constructor: CONSTRUCTOR_MODE.Int32Array,
                transformShader: init.getTransform32Shader(),
                sortShader: init.getSort32Shader(),
                untransformShader: init.getUntransform32Shader()
            };
        case "Uint32Array":
            return {
                mode: TRANSFORM_MODE.PASSTHROUGH,
                constructor: CONSTRUCTOR_MODE.Uint32Array,
                transformShader: init.getTransform32Shader(),
                sortShader: init.getSort32Shader(),
                untransformShader: init.getUntransform32Shader()
            };
        case "Float32Array":
            return {
                mode: TRANSFORM_MODE.FLOAT,
                constructor: CONSTRUCTOR_MODE.Float32Array,
                transformShader: init.getTransform32Shader(),
                sortShader: init.getSort32Shader(),
                untransformShader: init.getUntransform32Shader()
            };
        case "Float64Array":
            return {
                mode: TRANSFORM_MODE.FLOAT,
                constructor: CONSTRUCTOR_MODE.Float64Array,
                transformShader: init.getTransform64Shader(),
                sortShader: init.getSort64Shader(),
                untransformShader: init.getUntransform64Shader()
            };
        case "BigInt64Array":
            return {
                mode: TRANSFORM_MODE.INTEGER,
                constructor: CONSTRUCTOR_MODE.BigInt64Array,
                transformShader: init.getTransform64Shader(),
                sortShader: init.getSort64Shader(),
                untransformShader: init.getUntransform64Shader()
            };
        case "BigUint64Array":
            return {
                mode: TRANSFORM_MODE.PASSTHROUGH,
                constructor: CONSTRUCTOR_MODE.BigUint64Array,
                transformShader: init.getTransform64Shader(),
                sortShader: init.getSort64Shader(),
                untransformShader: init.getUntransform64Shader()
            };
    }
    throw new Error(`unsupported constructor.name: ${constructorName}`);
}
exports.getBitonicParameters = getBitonicParameters;
//# sourceMappingURL=parameters.js.map