"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bitonic_1 = require("./bitonic");
const radix_1 = require("./radix");
function sort(array) {
    if (!array.length)
        return array;
    switch (Object.prototype.toString.call(array)) {
        case "[object Int8Array]":
            return array.sort((a, b) => a - b);
        case "[object Uint8Array]":
            return array.sort((a, b) => a - b);
        case "[object Uint8ClampedArray]":
            return array.sort((a, b) => a - b);
        case "[object Int16Array]":
            return array.sort((a, b) => a - b);
        case "[object Uint16Array]":
            return array.sort((a, b) => a - b);
        case "[object Int32Array]":
            return bitonic_1.bitonicSort(array, "Int32Array");
        case "[object Uint32Array]":
            return bitonic_1.bitonicSort(array, "Uint32Array");
        case "[object Float32Array]":
            return bitonic_1.bitonicSort(array, "Float32Array");
        case "[object Float64Array]":
            return bitonic_1.bitonicSort(array, "Float64Array");
        case "[object BigInt64Array]":
            return bitonic_1.bitonicSort(array, "BigInt64Array");
        case "[object BigUint64Array]":
            return bitonic_1.bitonicSort(array, "BigUint64Array");
    }
    throw new Error(`${Object.prototype.toString.call(array)} is unsupported`);
}
exports.sort = sort;
function sortAsync(array) {
    return new Promise((resolve, reject) => {
        if (!array.length)
            return resolve();
        switch (Object.prototype.toString.call(array)) {
            case "[object Int8Array]":
                return radix_1.radixSortSignedAsync(new Uint8Array(array.buffer), 256)
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object Uint8Array]":
                return radix_1.radixSortAsync(new Uint8Array(array.buffer), 256)
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object Uint8ClampedArray]":
                return radix_1.radixSortAsync(new Uint8Array(array.buffer), 256)
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object Int16Array]":
                return radix_1.radixSortSignedAsync(new Uint16Array(array.buffer), 65536)
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object Uint16Array]":
                return radix_1.radixSortAsync(array, 65536)
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object Int32Array]":
                return bitonic_1.bitonicSortAsync(array, "Int32Array")
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object Uint32Array]":
                return bitonic_1.bitonicSortAsync(array, "Uint32Array")
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object Float32Array]":
                return bitonic_1.bitonicSortAsync(array, "Float32Array")
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object Float64Array]":
                return bitonic_1.bitonicSortAsync(array, "Float64Array")
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object BigInt64Array]":
                return bitonic_1.bitonicSortAsync(array, "BigInt64Array")
                    .then(() => resolve(array))
                    .catch(err => reject(err));
            case "[object BigUint64Array]":
                return bitonic_1.bitonicSortAsync(array, "BigUint64Array")
                    .then(() => resolve(array))
                    .catch(err => reject(err));
        }
        reject(new Error(`${Object.prototype.toString.call(array)} is unsupported`));
    });
}
exports.sortAsync = sortAsync;
//# sourceMappingURL=sort.js.map