"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parameters_1 = require("./parameters");
exports.isBigEndian = new Uint8Array(new Uint32Array([0x12345678]).buffer)[0] === 0x12;
exports.isLittleEndian = new Uint8Array(new Uint32Array([0x12345678]).buffer)[0] === 0x78;
class BitonicUniformGenerator {
    constructor(target, kind, emptyTexelCount) {
        this.target = target;
        this.parameters = parameters_1.getBitonicParameters(kind);
        this.emptyTexelCount = emptyTexelCount;
    }
    *generate() {
        for (let uniforms of this.generateTransformerUniforms())
            yield { shader: this.parameters.transformShader, uniforms: uniforms };
        for (let uniforms of this.generateSorterUniforms())
            yield { shader: this.parameters.sortShader, uniforms: uniforms };
        for (let uniforms of this.generateUntransformerUniforms())
            yield { shader: this.parameters.untransformShader, uniforms: uniforms };
    }
    *generateTransformerUniforms() {
        if (!exports.isLittleEndian || this.parameters.mode !== parameters_1.TRANSFORM_MODE.PASSTHROUGH) {
            const w = this.target.width;
            const x = w * w - this.emptyTexelCount;
            yield {
                u_bytes: this.target,
                u_width: w,
                u_mode: this.parameters.mode,
                u_endianness: exports.isLittleEndian ? 0 : 1,
                u_dataDelimX: Math.floor(x % w),
                u_dataDelimY: Math.floor(x / w)
            };
        }
    }
    *generateSorterUniforms() {
        const w = this.target.width;
        for (let rs = 4; rs <= 2 * w * w; rs *= 2) {
            for (let bs = rs / 2; bs > 1; bs /= 2) {
                yield {
                    u_bytes: this.target,
                    u_width: w,
                    u_blockSizeX: Math.floor(bs % w) > 0 ? Math.floor(bs % w) : w,
                    u_blockSizeY: Math.max(Math.floor(bs / w), 1),
                    u_regionSizeX: Math.floor(rs % w) > 0 ? Math.floor(rs % w) : w,
                    u_regionSizeY: Math.max(Math.floor(rs / w), 1)
                };
            }
        }
    }
    *generateUntransformerUniforms() {
        if (!exports.isLittleEndian || this.parameters.mode !== parameters_1.TRANSFORM_MODE.PASSTHROUGH) {
            yield {
                u_bytes: this.target,
                u_width: this.target.width,
                u_mode: this.parameters.mode,
                u_endianness: exports.isLittleEndian ? 0 : 1
            };
        }
    }
}
exports.BitonicUniformGenerator = BitonicUniformGenerator;
//# sourceMappingURL=uniforms.js.map